@@ -1,5 +1,4 @@
 package JavaScript::Duktape::XS;
-
 use strict;
 use warnings;
 use parent 'Exporter';
@@ -7,7 +6,7 @@ use parent 'Exporter';
 use Text::Trim qw(trim rtrim);
 use XSLoader;
 
-our $VERSION = '0.000065';
+our $VERSION = '0.000066';
 XSLoader::load( __PACKAGE__, $VERSION );
 
 our @EXPORT_OK = qw[];
@@ -37,9 +36,10 @@ sub _get_js_source_fragment {
 }
 
 sub parse_js_stacktrace {
-    my ($self, $stacktrace_lines, $desired_frames) = @_;
+    my ($self, $stacktrace_lines, $desired_frames, $priority_files) = @_;
 
     $desired_frames //= 1;
+    my %priority_files = map +( $_ => 1 ), @{ $priority_files // [] };
 
     # @contexts => [ {
     #   message => "undefined variable foo",
@@ -55,18 +55,26 @@ sub parse_js_stacktrace {
     #       ],
     #   }, {...} ]
     #   } ]
-    my @contexts;
+    my @contexts_hiprio;
+    my @contexts_normal;
     foreach my $line (@$stacktrace_lines) {
-        my @texts = split /\n/, trim($line);
+        $line = trim($line);
+        next unless $line;
+
+        my @texts = split /\n/, $line;
         my %context;
         $context{frames} = [];
         foreach my $text (@texts) {
             $text = trim($text);
             next unless $text;
 
+            # skip any of Duktape's internal functions
+            next if $text =~ m/^\s*at\s*.*internal$/;
+
             $context{message} = $text unless exists $context{message};
 
-            next unless $text =~ m/^\s*at\s*(\S*)\s*\((.*):([0-9]+)\)\s*$/;
+            next unless $text =~ m/^\s*at\s*(\S*)\s*\(([^:]*):([0-9]+)(:([0-9]+))?\)/;
+
             push @{ $context{frames} //= [] }, {
                 file => $2,
                 line => $3,
@@ -74,14 +82,23 @@ sub parse_js_stacktrace {
             last if scalar @{ $context{frames} } >= $desired_frames;
         }
         next unless exists $context{message};
+        next unless scalar @{ $context{frames} };
+        my $top_file = $context{frames}[0]{file};
+        next unless $top_file;
+
         _get_js_source_fragment(\%context);
-        push @contexts, \%context;
+        if ($priority_files && exists $priority_files{$top_file}) {
+            push @contexts_hiprio, \%context;
+        }
+        else {
+            push @contexts_normal, \%context;
+        }
+
     }
-    return \@contexts;
+    return [ @contexts_hiprio, @contexts_normal ];
 }
 
 1;
-
 __END__
 
 =pod
@@ -95,7 +112,7 @@ engine
 
 =head1 VERSION
 
-Version 0.000065
+Version 0.000066
 
 =head1 SYNOPSIS
 
@@ -267,6 +284,10 @@ structured information from it.  For each of the number of frames requested
 (default to 1), it gets the error message, the file name and line number where
 the error happened, and an array of lines surrounding the actual error message.
 
+The optional third parameter is an arrayref containing names of files that
+should be treated as top priority, meaning any errors pointing to those files
+will appear first in the returned stacktrace information.
+
 =head2 run_gc
 
 Run at least one round of the Duktape garbage collector, and return the number
